@@ -2,10 +2,23 @@
 import os
 import glob
 import logging
+import unittest
 logging.basicConfig()
 logging.getLogger().setLevel(logging.DEBUG)
 
-from BeautifulSoup import BeautifulSoup # required for html prettification
+try:
+    from bs4 import BeautifulSoup as _BeautifulSoup
+    class BeautifulSoup(_BeautifulSoup):
+        def prettify(self):
+            if self.html: self.html.unwrap()
+            if self.body: self.body.unwrap()
+            res = super(BeautifulSoup,self).prettify()
+            res = res.replace("/>"," />")
+            res = res.replace("  />"," />")
+            return str(res)
+except ImportError:
+    from BeautifulSoup import BeautifulSoup # required for html prettification
+from webob import Request, Response
 
 from sqlalchemy import *
 from sqlalchemy.orm import *
@@ -25,13 +38,13 @@ def ls(*args):
     files.sort()
     for f in files:
         if os.path.isdir(f):
-            print 'D %s' % os.path.basename(f)
+            print('D %s' % os.path.basename(f))
         else:
-            print '- %s' % os.path.basename(f)
+            print('- %s' % os.path.basename(f))
 
 def cat(*args):
     filename = os.path.join(os.path.dirname(__file__), *args)
-    print open(filename).read()
+    print(open(filename).read())
 
 def session_mapper(scoped_session):
     def mapper(cls, *arg, **kw):
@@ -39,6 +52,35 @@ def session_mapper(scoped_session):
         return sqla_mapper(cls, *arg, **kw)
     return mapper
 
+def application(model, fieldset=None):
+    def app(environ, start_response):
+        tmpl = u'''<DOCTYPE !html>
+        <html><body><form method="POST" action="" enctype="multipart/form-data">
+        %s
+        <input type="submit" id="submit" name="submit" />
+        </form></body></html>'''
+        req = Request(environ)
+        resp = Response()
+        if fieldset is None:
+            fs = FieldSet(model)
+        else:
+            fs = fieldset.bind(model)
+        if req.method == 'POST':
+            if fieldset is None:
+                fs = fs.bind(request=req)
+            else:
+                fs = fs.bind(model=model, request=req)
+            if fs.validate():
+                fs.sync()
+                fs.readonly = True
+                resp.text = tmpl % ('<b>OK<b>' + fs.render(),)
+            else:
+                resp.text = tmpl % fs.render()
+        else:
+            resp.text = tmpl % fs.render()
+        return resp(environ, start_response)
+    return app
+
 
 engine = create_engine('sqlite://')
 Session = scoped_session(sessionmaker(autoflush=False, bind=engine))
@@ -164,6 +206,26 @@ class Recursive(Base):
     parent_id = Column(Integer, ForeignKey("recursives.id"))
     parent = relation('Recursive', primaryjoin=parent_id==id, uselist=False, remote_side=parent_id)
 
+class RecursiveBase(Base):
+    __tablename__ = 'recursivebase'
+    __mapper_args__ = {'polymorphic_on': 'typ_id'}
+
+    id = Column(Integer, primary_key=True)
+    typ_id = Column(Integer, nullable=False)
+    foo = Column(Text, nullable=True)
+    parent_id = Column(Integer, ForeignKey(id))
+    parent = relation('Recursive', primaryjoin=parent_id==id, uselist=False, remote_side=parent_id)
+
+class RecursiveChild(RecursiveBase):
+    __tablename__ = 'recursivechild'
+    __mapper_args__ = {'polymorphic_identity': 42}
+    id = Column(None, ForeignKey(RecursiveBase.id, ondelete='CASCADE', onupdate='RESTRICT'), primary_key=True)
+    bar = Column(Text, nullable=True)
+    @classmethod
+    def __declare_last__(cls):
+        if not hasattr(cls,'owner'):
+            cls.owner = cls.parent
+
 class Synonym(Base):
     __tablename__ = 'synonyms'
     id = Column(Integer, primary_key=True)
@@ -174,6 +236,17 @@ class Synonym(Base):
         return self._foo
     foo = synonym('_foo', descriptor=property(_get_foo, _set_foo))
 
+class AliasedRecursive(Base):
+    __tablename__ = 'aliasedrecursive'
+    id = Column(Integer, primary_key=True)
+    foo = Column(Text, nullable=True)
+    parent_id = Column(Integer, ForeignKey(id))
+    parent = relation('Recursive', primaryjoin=parent_id==id, uselist=False, remote_side=parent_id)
+    @classmethod
+    def __declare_last__(cls):
+        if not hasattr(cls,'owner'):
+            cls.owner = cls.parent
+
 class OTOChild(Base):
     __tablename__ = 'one_to_one_child'
     id = Column(Integer, primary_key=True)
@@ -181,6 +254,7 @@ class OTOChild(Base):
 
     def __unicode__(self):
         return self.baz
+    __str__ = __unicode__
 
     def __repr__(self):
         return '<OTOChild %s>' % self.baz
@@ -198,6 +272,7 @@ class Order(Base):
     quantity = Column(Integer, nullable=False)
     def __unicode__(self):
         return 'Quantity: %s' % self.quantity
+    __str__ = __unicode__
     def __repr__(self):
         return '<Order for user %s: %s>' % (self.user_id, self.quantity)
 
@@ -209,6 +284,7 @@ class OptionalOrder(Base): # the user is optional, not the order
     user = relation('User')
     def __unicode__(self):
         return 'Quantity: %s' % self.quantity
+    __str__ = __unicode__
     def __repr__(self):
         return '<OptionalOrder for user %s: %s>' % (self.user_id, self.quantity)
 
@@ -223,6 +299,7 @@ class User(Base):
     orders_dl = dynamic_loader(Order)
     def __unicode__(self):
         return self.name
+    __str__ = __unicode__
     def __repr__(self):
         return '<User %s>' % self.name
     def __html__(self):
@@ -356,9 +433,7 @@ else:
     raise ImportError('mako is required for testing')
 
 def pretty_html(html):
-    if isinstance(html, unicode):
-        html = html.encode('utf-8')
-    soup = BeautifulSoup(str(html))
+    soup = BeautifulSoup(html)
     return soup.prettify().strip()
 
 class FieldSet(DefaultFieldSet):
@@ -403,18 +478,11 @@ def configure_and_render(fs, **options):
     fs.configure(**options)
     return fs.render()
 
-if not hasattr(__builtins__, 'sorted'):
-    # 2.3 support
-    def sorted(L, key=lambda a: a):
-        L = list(L)
-        L.sort(lambda a, b: cmp(key(a), key(b)))
-        return L
-
 class ImgRenderer(TextFieldRenderer):
     def render(self, *args, **kwargs):
         return '<img src="%s">' % self.value
 
-import fake_module
+from formalchemy.tests import fake_module
 fake_module.__dict__.update({
         'fs': FieldSet(User, session=session),
         })
